//TrafficLights.ino - Demonstration of Led, Switch and Timer library.
//Copyright (c) 2018, Julie Gonzales VK3FOWL and Joe Gonzales VK3YSP.
//Released under the GNU General Public License.

#include "led.h"
#include "switch.h"
#include "timer.h"

/* Traffic Lights State Transition Table
  State Road1 Road2 Cross1  Cross2  Time  Input Next
  GRWD  G1    R2    W1      D2      10000 P2    GRFD
  GRFD  G1    R2    F1      D2      3000        GRDD
  GRDD  G1    R2    D1      D2      2000        YRDD
  YRDD  Y1    R2    D1      D2      2000        RRDDa
  RRDDa R1    R2    D1      D2      1000        RGDW
  RGDW  R1    G2    D1      W2      10000 P1    RGDF
  RGDF  R1    G2    D1      F2      3000        RGDD
  RGDD  R1    G2    D1      D2      2000        RYDD
  RYDD  R1    Y2    D1      D2      2000        RRDDb
  RRDDb R1    R2    D1      D2      1000        GRWD
*/

//Declare Traffic Light States
enum State {GRWD, GRFD, GRDD, YRDD, RRDDa, RGDW, RGDF, RGDD, RYDD, RRDDb};
State state = GRWD, lastState = RRDDb;

//Declare Red, Yellow, Green, Don'twalk and Walk LEDs for roads 1 and 2
Led R1(10), Y1(16), G1(14), R2(9), Y2(8), G2(7), D1(15), W1(18), D2(6), W2(5);

//Declare Pedestrian Crossing press-button switches for roads 1 and 2
Switch S1(20, Press), S2(3, Press);

//Declare timer
Timer t1;

void off() {
  //Turn off all lights (just prior to turning four lights on)
  R1.off(); Y1.off(); G1.off();
  R2.off(); Y2.off(); G2.off();
  D1.off(); W1.off();
  D2.off(); W2.off();
}

void initState(long period) {
  //Initialise the state
  if (lastState != state) { //Only run this on entering the state for the first time
    lastState = state;      //Update the last state
    t1.reset(period);       //Reset the timer
    off();                  //Turn off all lights
  }
}

void setup() {
}

void loop() {
  //Traffic light state machine
  switch (state) {
    case GRWD:
      //GRWD  G1    R2    W1      D2      10000        GRFD
      initState(5000);                                //Entry
      G1.on(); R2.on(); W1.on(); D2.on();              //Outputs
      if (t1.timeout() || S2.state()) state = GRFD;    //Exit
      break;
    case GRFD:
      //GRFD  G1    R2    F1      D2      3000        GRDD
      initState(3000);                                //Entry
      G1.on(); R2.on(); D1.flash(200, 200); D2.on();  //Outputs
      if (t1.timeout()) state = GRDD;                 //Exit
      break;
    case GRDD:
      //GRDD  G1    R2    D1      D2      2000        YRDD
      initState(2000);                                //Entry
      G1.on(); R2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = YRDD;                 //Exit
      break;
    case YRDD:
      //YRDD  Y1    R2    D1      D2      2000        RRDDa
      initState(2000);                                //Entry
      Y1.on(); R2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = RRDDa;                //Exit
      break;
    case RRDDa:
      //RRDDa R1    R2    D1      D2      1000        RGDW
      initState(1000);                                //Entry
      R1.on(); R2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = RGDW;                 //Exit
      break;
    case RGDW:
      //RGDW  R1    G2    D1      W2      10000        RGDF
      initState(5000);                                //Entry
      R1.on(); G2.on(); D1.on(); W2.on();              //Outputs
      if (t1.timeout() || S1.state()) state = RGDF;    //Exit 
      break;
    case RGDF:
      //RGDF  R1    G2    D1      F2      3000        RGDD
      initState(3000);                                //Entry
      R1.on(); G2.on(); D1.on(); D2.flash(200, 200);  //Outputs
      if (t1.timeout()) state = RGDD;                 //Exit
      break;
    case RGDD:
      //RGDD  R1    G2    D1      D2      2000        RYDD
      initState(2000);                                //Entry
      R1.on(); G2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = RYDD;                 //Exit
      break;
    case RYDD:
      //RYDD  R1    Y2    D1      D2      2000        RRDDb
      initState(2000);                                //Entry
      R1.on(); Y2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = RRDDb;                //Exit
      break;
    case RRDDb:
      //RRDDb R1    R2    D1      D2      1000        GRWD
      initState(1000);                                //Entry
      R1.on(); R2.on(); D1.on(); D2.on();             //Outputs
      if (t1.timeout()) state = GRWD;                 //Exit
      break;
  }
}
